@@ -1,11 +1,11 @@
 /* xgetdomainname.c -- Return the NIS domain name, without size limitations.
-   Copyright (C) 1992, 1996, 2000, 2001, 2003, 2004 Free Software Foundation,
-   Inc.
+   Copyright (C) 1992, 1996, 2000-2001, 2003-2004, 2006, 2008-2013 Free
+   Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,20 +13,17 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Based on xgethostname.c, written by Jim Meyering.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 /* Specification.  */
 #include "xgetdomainname.h"
 
 /* Get getdomainname.  */
-#include "getdomainname.h"
+#include <unistd.h>
 
 /* Get errno.  */
 #include <errno.h>
@@ -64,14 +61,14 @@ xgetdomainname (void)
       domainname[k] = '\0';
       err = getdomainname (domainname, size);
       if (err >= 0 && domainname[k] == '\0')
-	break;
+        break;
       else if (err < 0 && errno != EINVAL)
-	{
-	  int saved_errno = errno;
-	  free (domainname);
-	  errno = saved_errno;
-	  return NULL;
-	}
+        {
+          int saved_errno = errno;
+          free (domainname);
+          errno = saved_errno;
+          return NULL;
+        }
       size *= 2;
       domainname = xrealloc (domainname, size);
     }
