@@ -1,5 +1,5 @@
 /* cfg.h --- Configuration file functions.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2013 Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -117,7 +117,6 @@ shishi_cfg (Shishi * handle, const char *option)
   char *opt = option ? xstrdup (option) : NULL;
   char *p = opt;
   char *value;
-  char *realm = NULL;
   int res;
   size_t i;
 
@@ -182,6 +181,10 @@ shishi_cfg (Shishi * handle, const char *option)
 
 	case REALM_KDC_OPTION:
 	  {
+	    struct Shishi_realminfo *ri;
+	    char *realm = NULL;
+	    char *protstr;
+	    int transport = UDP;
 	    int add_realm = 1;
 
 	    realm = xstrdup (value);
@@ -194,6 +197,7 @@ shishi_cfg (Shishi * handle, const char *option)
 		      free (handle->realminfos[i].kdcaddresses);
 		      handle->realminfos[i].kdcaddresses = NULL;
 		      handle->realminfos[i].nkdcaddresses = 0;
+		      ri = &handle->realminfos[i];
 		      add_realm = 0;
 		    }
 		  break;
@@ -206,19 +210,71 @@ shishi_cfg (Shishi * handle, const char *option)
 		memset (&handle->realminfos[handle->nrealminfos], 0,
 			sizeof (handle->realminfos[handle->nrealminfos]));
 		handle->realminfos[handle->nrealminfos].name = realm;
+		ri = &handle->realminfos[handle->nrealminfos];
 		handle->nrealminfos++;
 	      }
+	    if ((protstr = strchr (p, '/')))
+	      {
+		*protstr = '\0';
+		protstr++;
+		if (strcasecmp (protstr, "udp") == 0)
+		  transport = UDP;
+		else if (strcasecmp (protstr, "tcp") == 0)
+		  transport = TCP;
+		else if (strcasecmp (protstr, "tls") == 0)
+		  transport = TLS;
+		else
+		  shishi_warn (handle,
+			       "Ignoring unknown KDC transport: %s",
+				   protstr);
+	      }
+
+	    ri->kdcaddresses = xrealloc (ri->kdcaddresses,
+					 (ri->nkdcaddresses + 1) *
+					   sizeof (*ri->kdcaddresses));
+	    ri->kdcaddresses[ri->nkdcaddresses].transport = transport;
+	    ri->kdcaddresses[ri->nkdcaddresses].hostname = xstrdup (p);
+	    if ((protstr = strchr (value, ':')))
+	      {
+		*protstr = '\0';
+		protstr++;
+		ri->kdcaddresses[ri->nkdcaddresses].port = protstr;
+	      }
+	    else
+	      ri->kdcaddresses[ri->nkdcaddresses].port = NULL;
+	    ri->nkdcaddresses++;
+
+	    p = NULL;	/* Done with suboptions.  */
 	  }
 	  break;
 
 	case SERVER_REALM_OPTION:
 	  {
 	    struct Shishi_realminfo *ri;
-	    ri = _shishi_realminfo_new (handle, value);
-	    ri->serverwildcards = xrealloc (ri->serverwildcards,
-					    ++ri->nserverwildcards *
-					    sizeof (*ri->serverwildcards));
-	    ri->serverwildcards[ri->nserverwildcards - 1] = xstrdup (value);
+	    char *subopts, *part, *next;
+
+	    if (!p || (*p == 0))
+	      {
+		shishi_warn (handle, "Empty server-realm for '%s'.", value);
+		break;
+	      }
+
+	    ri = _shishi_realminfo_new (handle, xstrdup (value));
+
+	    part = subopts = xstrdup (p);	/* List of patterns.  */
+	    while (part && *part)
+	      {
+		next = strchr (part, ',');
+		if (next)
+		  *(next++) = '\0';
+
+		ri->serverwildcards = xrealloc (ri->serverwildcards,
+						++ri->nserverwildcards *
+						sizeof (*ri->serverwildcards));
+		ri->serverwildcards[ri->nserverwildcards - 1] = xstrdup (part);
+		part = next;
+	      }
+	    p = NULL;	/* Done with suboptions.  */
 	  }
 	  break;
 
@@ -275,47 +331,6 @@ shishi_cfg (Shishi * handle, const char *option)
 	case -1:
 	  if (!value)
 	    break;
-	  for (i = 0; i < handle->nrealminfos; i++)
-	    if (realm && strcmp (handle->realminfos[i].name, realm) == 0)
-	      {
-		struct Shishi_realminfo *ri = &handle->realminfos[i];
-		char *protstr;
-		int transport = UDP;
-
-		if ((protstr = strchr (value, '/')))
-		  {
-		    *protstr = '\0';
-		    protstr++;
-		    if (strcasecmp (protstr, "udp") == 0)
-		      transport = UDP;
-		    else if (strcasecmp (protstr, "tcp") == 0)
-		      transport = TCP;
-		    else if (strcasecmp (protstr, "tls") == 0)
-		      transport = TLS;
-		    else
-		      shishi_warn (handle,
-				   "Ignoring unknown KDC transport: %s",
-				   protstr);
-		  }
-
-		ri->kdcaddresses = xrealloc (ri->kdcaddresses,
-					     (ri->nkdcaddresses + 1) *
-					     sizeof (*ri->kdcaddresses));
-		ri->kdcaddresses[ri->nkdcaddresses].transport = transport;
-		ri->kdcaddresses[ri->nkdcaddresses].hostname =
-		  xstrdup (value);
-		if ((protstr = strchr (value, ':')))
-		  {
-		    *protstr = '\0';
-		    protstr++;
-		    ri->kdcaddresses[ri->nkdcaddresses].port = protstr;
-		  }
-		else
-		  ri->kdcaddresses[ri->nkdcaddresses].port = NULL;
-		ri->nkdcaddresses++;
-	      }
-	  if (realm)
-	    break;
 	  /* fall through */
 
 	default:
@@ -584,7 +599,7 @@ shishi_cfg_clientkdcetype_fast (Shishi * handle)
 
 /**
  * shishi_cfg_clientkdcetype_set:
- * @handle: Shishi library handle create by shishi_init().
+ * @handle: Shishi library handle created by shishi_init().
  * @value: string with encryption types.
  *
  * Set the "client-kdc-etypes" configuration option from given string.
@@ -592,20 +607,23 @@ shishi_cfg_clientkdcetype_fast (Shishi * handle)
  * by comma or whitespace, e.g. "aes256-cts-hmac-sha1-96
  * des3-cbc-sha1-kd des-cbc-md5".
  *
- * Return value: Return SHISHI_OK iff successful.
+ * Return value: Returns SHISHI_OK if successful.
  **/
 int
 shishi_cfg_clientkdcetype_set (Shishi * handle, char *value)
 {
   char *ptrptr;
-  char *val;
+  char *val, *tmpvalue;
   int i;
   int tot = 0;
+  int rc = SHISHI_INVALID_ARGUMENT;
 
   if (value == NULL || *value == '\0')
     return SHISHI_OK;
 
-  for (i = 0; (val = strtok_r (i == 0 ? value : NULL, ", \t", &ptrptr)); i++)
+  tmpvalue = xstrdup (value);
+
+  for (i = 0; (val = strtok_r (i == 0 ? tmpvalue : NULL, ", \t", &ptrptr)); i++)
     {
       int etype = shishi_cipher_parse (val);
 
@@ -621,35 +639,41 @@ shishi_cfg_clientkdcetype_set (Shishi * handle, char *value)
 	  handle->clientkdcetypes = new;
 	  handle->clientkdcetypes[tot - 1] = etype;
 	  handle->nclientkdcetypes = tot;
+	  rc = SHISHI_OK;	/* At least one valid type.  */
 	}
     }
 
-  return SHISHI_OK;
+  free (tmpvalue);
+
+  return rc;
 }
 
 /**
  * shishi_cfg_authorizationtype_set:
- * @handle: Shishi library handle create by shishi_init().
+ * @handle: Shishi library handle created by shishi_init().
  * @value: string with authorization types.
  *
  * Set the "authorization-types" configuration option from given string.
  * The string contains authorization types (integer or names) separated
  * by comma or whitespace, e.g. "basic k5login".
  *
- * Return value: Return SHISHI_OK iff successful.
+ * Return value: Returns SHISHI_OK if successful.
  **/
 int
 shishi_cfg_authorizationtype_set (Shishi * handle, char *value)
 {
   char *ptrptr;
-  char *val;
+  char *val, *tmpvalue;
   int i;
   int tot = 0;
+  int rc = SHISHI_INVALID_ARGUMENT;
 
   if (value == NULL || *value == '\0')
     return SHISHI_OK;
 
-  for (i = 0; (val = strtok_r (i == 0 ? value : NULL, ", \t", &ptrptr)); i++)
+  tmpvalue = xstrdup (value);
+
+  for (i = 0; (val = strtok_r (i == 0 ? tmpvalue : NULL, ", \t", &ptrptr)); i++)
     {
       int atype = shishi_authorization_parse (val);
 
@@ -666,8 +690,11 @@ shishi_cfg_authorizationtype_set (Shishi * handle, char *value)
 	  handle->authorizationtypes = new;
 	  handle->authorizationtypes[tot - 1] = atype;
 	  handle->nauthorizationtypes = tot;
+	  rc = SHISHI_OK;	/* At least one valid type.  */
 	}
     }
 
-  return SHISHI_OK;
+  free (tmpvalue);
+
+  return rc;
 }
