@@ -1,5 +1,5 @@
 /* encticketpart.c --- Encrypted ticket part handling.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2013 Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -212,27 +212,15 @@ shishi_encticketpart_cname_set (Shishi * handle,
 {
   int res;
 
+  res = shishi_principal_set (handle, encticketpart, "cname", principal);
+  if (res != SHISHI_OK)
+    return res;
+
   res = shishi_asn1_write_uint32 (handle, encticketpart,
 				  "cname.name-type", name_type);
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, encticketpart,
-			   "cname.name-string", NULL, 0);
-  if (res != SHISHI_OK)
-    return res;
-
-  res = shishi_asn1_write (handle, encticketpart,
-			   "cname.name-string", "NEW", 1);
-  if (res != SHISHI_OK)
-    return res;
-
-  res = shishi_asn1_write (handle, encticketpart,
-			   "cname.name-string.?1",
-			   principal, strlen (principal));
-  if (res != SHISHI_OK)
-    return res;
-
   return SHISHI_OK;
 }
 
