@@ -1,5 +1,5 @@
 /* error.c --- Error handling functions.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2013 Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -47,21 +47,27 @@ static const struct shishi_error_msgs _shishi_error_messages[] = {
    N_("Client name value differ between request and reply.")},
   {SHISHI_NONCE_MISMATCH,
    N_("Replay protection value (nonce) differ between request and reply.")},
-  {SHISHI_TICKET_BAD_KEYTYPE,
-   N_("Keytype used to encrypt ticket doesn't match provided key. "
-      "This usually indicates an internal application error.")},
-  {SHISHI_CRYPTO_INTERNAL_ERROR,
-   N_("Internal error in low-level crypto routines.")},
+  {SHISHI_TGSREP_BAD_KEYTYPE,
+   N_("Incorrect key type used in TGS reply.")},
+  {SHISHI_KDCREP_BAD_KEYTYPE,
+   N_("Incorrect key type used in reply from KDC.")},
+  {SHISHI_APREP_BAD_KEYTYPE,
+   N_("Incorrect key type used in AP reply.")},
+  {SHISHI_APREP_VERIFY_FAILED,
+   N_("Failed verification of AP reply.")},
+  {SHISHI_APREQ_BAD_KEYTYPE,
+   N_("Incorrect key type used in AP request.")},
+  {SHISHI_TOO_SMALL_BUFFER,
+   N_("Provided buffer was too small.")},
+  {SHISHI_DERIVEDKEY_TOO_SMALL,
+   N_("Derived key material is too short to be applicable.")},
+  {SHISHI_KEY_TOO_LARGE,
+   N_("The key is too large to be usable.")},
   {SHISHI_CRYPTO_ERROR,
    N_("Low-level cryptographic primitive failed.  This usually indicates "
       "bad password or data corruption.")},
-  {SHISHI_RECVFROM_ERROR,
-   N_("Error receiving data from server")},
-  {SHISHI_KDC_TIMEOUT,
-   N_("Timedout talking to KDC. This usually indicates a network "
-      "or KDC address problem.")},
-  {SHISHI_KDC_NOT_KNOWN_FOR_REALM,
-   N_("No KDC for realm known.")},
+  {SHISHI_CRYPTO_INTERNAL_ERROR,
+   N_("Internal error in low-level crypto routines.")},
   {SHISHI_SOCKET_ERROR,
    N_("The system call socket() failed.  This usually indicates that "
       "your system does not support the socket type.")},
@@ -70,19 +76,72 @@ static const struct shishi_error_msgs _shishi_error_messages[] = {
       "insufficient permissions.")},
   {SHISHI_SENDTO_ERROR,
    N_("The system call sendto() failed.")},
+  {SHISHI_RECVFROM_ERROR,
+   N_("Error receiving data from server")},
   {SHISHI_CLOSE_ERROR,
    N_("The system call close() failed.")},
+  {SHISHI_KDC_TIMEOUT,
+   N_("Timed out talking to KDC. This usually indicates a network "
+      "or KDC address problem.")},
+  {SHISHI_KDC_NOT_KNOWN_FOR_REALM,
+   N_("No KDC known for given realm.")},
+  {SHISHI_TTY_ERROR,
+   N_("No TTY assigned to process.")},
   {SHISHI_GOT_KRBERROR,
-   N_("Server replied with an error message to request.")},
+   N_("Server replied to the request with an error message.")},
+  {SHISHI_HANDLE_ERROR,
+   N_("Failure to use handle.  Missing handle, or misconfigured.")},
   {SHISHI_INVALID_TKTS,
-   N_("Ticketset not initialized.  This usually indicates an internal "
+   N_("Ticket set not initialized.  This usually indicates an internal "
       "application error.")},
+  {SHISHI_TICKET_BAD_KEYTYPE,
+   N_("Key type used to encrypt ticket doesn't match provided key. "
+      "This usually indicates an internal application error.")},
+  {SHISHI_INVALID_KEY,
+   N_("Reference to invalid encryption key.")},
   {SHISHI_APREQ_DECRYPT_FAILED,
    N_("Could not decrypt AP-REQ using provided key. "
       "This usually indicates an internal application error.")},
   {SHISHI_TICKET_DECRYPT_FAILED,
    N_("Could not decrypt Ticket using provided key. "
       "This usually indicates an internal application error.")},
+  {SHISHI_INVALID_TICKET,
+   N_("Invalid ticked passed in call.")},
+  {SHISHI_OUT_OF_RANGE,
+   N_("Argument lies outside of valid range.")},
+  {SHISHI_ASN1_NO_ELEMENT,
+   N_("The ASN.1 structure does not contain the indicated element.")},
+  {SHISHI_SAFE_BAD_KEYTYPE,
+   N_("Attempted access to non-existent key type.")},
+  {SHISHI_SAFE_VERIFY_FAILED,
+   N_("Verification failed on either side.")},
+  {SHISHI_PKCS5_INVALID_PRF,
+   N_("Invalid PKCS5 descriptor.")},
+  {SHISHI_PKCS5_INVALID_ITERATION_COUNT,
+   N_("Invalid claim of iteration count in PKCS5 descriptor.")},
+  {SHISHI_PKCS5_INVALID_DERIVED_KEY_LENGTH,
+   N_("Derived key length is incorrect for PKCS5 descriptor.")},
+  {SHISHI_PKCS5_DERIVED_KEY_TOO_LONG,
+   N_("Derived key is too long for PKCS5 descriptor.")},
+  {SHISHI_INVALID_PRINCIPAL_NAME,
+   N_("Principal name syntax error.")},
+  {SHISHI_INVALID_ARGUMENT,
+   N_("Invalid argument passed in call.  Wrong or unknown value.")},
+  {SHISHI_ASN1_NO_VALUE,
+   N_("The indicated ASN.1 element does not carry a value.")},
+  {SHISHI_CONNECT_ERROR,
+   N_("Connection attempt failed.  Try again, or check availability.")},
+  {SHISHI_VERIFY_FAILED,
+   N_("Verification failed on either side.")},
+  {SHISHI_PRIV_BAD_KEYTYPE,
+   N_("The private key uses an incompatible encryption type.")},
+  {SHISHI_FILE_ERROR,
+   N_("The desired file could not be accessed.  Check permissions.")},
+  {SHISHI_ENCAPREPPART_BAD_KEYTYPE,
+   N_("The present AP reply specifies an inpermissible key type.")},
+  {SHISHI_GETTIMEOFDAY_ERROR,
+   N_("A request for present time of day has failed. "
+      "This is usually internal, but a valid time is imperative for us.")},
   {SHISHI_KEYTAB_ERROR,
    N_("Failed to parse keytab file")},
   {SHISHI_CCACHE_ERROR,
