@@ -1,5 +1,5 @@
 /* internal.h --- Internal header file for Shishi.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2013 Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -49,7 +49,7 @@
 
 #include <netdb.h>
 
-#if defined HAVE_DECL_H_ERRNO && !HAVE_DECL_H_ERRNO
+#if !HAVE_DECL_H_ERRNO
 /*extern int h_errno;*/
 #endif
 
@@ -63,13 +63,6 @@
 #endif
 #include <arpa/inet.h>
 
-#ifdef HAVE_ARPA_NAMESER_H
-#include <arpa/nameser.h>
-#endif
-#ifdef HAVE_RESOLV_H
-#include <resolv.h>
-#endif
-
 #ifdef HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
