@@ -1,5 +1,5 @@
 /* shishid.c --- Shishi Key Distribution Center daemon.
- * Copyright (C) 2002-2012 Simon Josefsson
+ * Copyright (C) 2002-2013 Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -63,6 +63,17 @@ kdc_listen (void)
 	  goto error;
 	}
 
+#ifdef IPV6_V6ONLY
+      if (ls->ai.ai_family == AF_INET6)
+	{
+	  yes = 1;
+	  if (setsockopt (ls->sockfd, IPPROTO_IPV6, IPV6_V6ONLY,
+			  (char *) &yes, sizeof (yes)) < 0)
+	    error (0, errno, "Cannot restrict %s to AF_INET6 only",
+		   ls->addrname);
+	}
+#endif
+
       yes = 1;
       if (setsockopt (ls->sockfd, SOL_SOCKET, SO_REUSEADDR,
 		      (char *) &yes, sizeof (yes)) < 0)
@@ -336,10 +347,11 @@ doit (void)
     const char *shortname = (slash != NULL ? slash + 1 : program_name);
 
 #ifdef LOG_PERROR
-    openlog (shortname, LOG_CONS | LOG_PERROR, LOG_DAEMON);
-#else
-    openlog (shortname, LOG_CONS, LOG_DAEMON);
+    if (arg.verbose_given > 0)
+      openlog (shortname, LOG_CONS | LOG_PERROR, LOG_AUTH);
+    else
 #endif
+      openlog (shortname, LOG_CONS, LOG_AUTH);
   }
 
   kdc_setuid ();
